
This is a set of functions that create and maintain an .ini file
(Prog.ini in this example.)

Uploaded by Gy. Gorog, 100324,332. Public domain: good luck!

The only difference from a usual .ini file is that I felt safer to
follow the Hungarian notation in the .ini file too; that is, an item's
first letter refers to the type of the variable to read in.
( After all I'm a Hungarian...)

This code, as is, requires Tools 3.0.

The function IniItem() reads and optionally sets an item;
if unable to find that item and no new value is given, it returns NIL.
Therefore, I always DEFAULT the variable.

The first example function reads the following section in the .ini file:

[Label]

cPrinter=InterMec
nCom=2
nBaud=9600
cParit=E
nBits=7
nStop=1

******************************************************************************

STATIC cIni
STATIC CrLf  // Chr(13) + Chr(10)

******************************************************************************
FUNCTION LabelPrint( nLabels )

// Demonstrates how to read items from the .ini file to drive a label printer.

STATIC cLabel

LOCAL cLFile  := IniItem( "Label", "cLFile"   )
LOCAL cPrinter:= IniItem( "Label", "cPrinter" )
LOCAL nCom    := IniItem( "Label", "nCom"     )
LOCAL nBaud   := IniItem( "Label", "nBaud"    )
LOCAL cParit  := IniItem( "Label", "cParit"   )
LOCAL nBits   := IniItem( "Label", "nBits"    )
LOCAL nStop   := IniItem( "Label", "nStop"    )
LOCAL cSerial := IniItem( "Label", "cSerial"  )

// Read cLabel, preferably once in a lifetime
Default cLFile To "Label.Prt"
Default cLabel To MemoRead( cLFile )

IF Empty( cLabel )
   RETURN .F.
ENDIF

// Printer values
Default cPrinter To "InterMec"
Default nCom     To    2
Default nBaud    To 9600
Default cParit   To   "E"
Default nBits    To    7
Default nStop    To    1

// Print
IF Com_Open( nCom )

   IF Com_Init( nCom, nBaud, cParit, nBits, nStop )

      // Serial no merged labels
      FOR n := 1 TO nLabels
	WHILE !Empty( Com_Send( nCom, cLPr )) .AND. InKey() # K_ESC; END
      NEXT
   ENDIF

   Com_Close( nCom )
ENDIF

RETURN .T.
// eof LabelPrint()
*****************************************************************************
FUNCTION SetPrinter()

// Demonstrates how to set an item

LOCAL cPrinter := IniItem( "Printer", "cPrType" )
Default cPrinter To "Epson"

IF cPrinter == "Epson"
   nPrinter := 1
ELSEIF cPrinter == "HP LaserJet"
   nPrinter := 2
ENDIF

nPrinter := MenuBox( 10, 30, {"EPSON"  , ;
			      "LASER"" } )

IF !Empty( nPrinter )
   IF     nPrinter == 1
     cPrinter := "Epson"
   ELSEIF nPrinter == 2
     cPrinter := "HP LaserJet"
   ENDIF

   // here set new item
   IniItem( "Printer", "cPrType", cPrinter )

ENDIF

RETURN !Empty( nPrinter )
// eof SetPrinter()
*****************************************************************************
***************************** INI functions *********************************
*****************************************************************************
FUNCTION IniOpen()

PUBLIC cIni := MemoRead( "Prog.Ini" )

RETURN .T.
// eof IniOpen()
*****************************************************************************
FUNCTION IniClose()

// some formatting
cIni := StrTran( cIni, CrLf + CrLf + "[", CrLf + "[")
cIni := StrTran( cIni, CrLf + "[", CrLf + CrLf + "[")

RETURN MemoWrit( "Prog.Ini", cIni )
// eof IniClose()
*****************************************************************************
FUNCTION IniItem( cSecName, cItem, NewValue )

// Modify or get previous value

LOCAL cType := Upper( Left( cItem, 1 ))
LOCAL OldValue, lFound := .F.
LOCAL cOldSec, cNewSec, cOldValue, cNewValue
LOCAL lMod := NewValue # NIL
LOCAL n, nLineWidth, nLine, nLines, cOldLine, cNewLine

cSecName := "[" + AllTrim( cSecName ) + "]"

IF lMod  // Modify value

  IF Empty( cOldSec := AfterAtNum( cSecName, cIni ))
     // section did not exist; create section
     cNewSec := CrLf + CrLf + cSecName + CrLf + CrLf
  ELSE
     // section exists; see if item exists
     cNewSec := cOldSec := cSecName + IF( Empty( BeforAtNum( "[", cOldSec, 1 )), cOldSec, BeforAtNum( "[", cOldSec, 1 ))
     nLines  := MLCount ( cOldSec, nLineWidth := MaxLine( cOldSec ))
     FOR n := 1 TO nLines
       cOldLine := AllTrim( MemoLine( cOldSec, nLineWidth, n ))
       // comment lines begin with a non-letter character
       IF IsAlpha( Left( cOldLine, 1 )) .AND. ;
	  Left( cOldLine, Len( cItem )) == cItem
	  IF Empty( cOldValue := AfterAtNum( "=", cOldLine ))
	    // item did not exist; add new item
	    cNewSec   += cItem + "=" + Ini2Str( NewValue ) + CrLf
	  ELSE
	    // item exists; read old value and put in new one
	    OldValue := Str2Ini( cOldValue, cType )
	    cNewSec := BeforAtNum( CrLf, cOldSec, n-1 ) + CrLf + ;
		       cItem + "=" + Ini2Str( NewValue) + CrLf + ;
		       AfterAtNum( CrLf, cOldSec, n   )
	  ENDIF
	  n := nLines; lFound := .T.
	ENDIF
     NEXT
  ENDIF

  IF !lFound
    // add new item to section
    cNewSec += cItem + "=" + Ini2Str( NewValue ) + CrLf
  ENDIF

  IF Empty( cOldSec )
    // add section to ini
    cIni += cNewSec + CrLf
  ELSE
    // replace old section with new
    cIni := StrTran( cIni, cOldSec, cNewSec )
  ENDIF

ELSE  // Read value only

  IF Empty( cOldSec := AfterAtNum( cSecName, cIni ))
     // nothing to read; OldValue remains NIL
  ELSE
     // section exists; see if item exists
     cOldSec := cSecName + IF( Empty( BeforAtNum( "[", cOldSec, 1 )), cOldSec, BeforAtNum( "[", cOldSec, 1 ))
     nLines  := MLCount ( cOldSec, nLineWidth := MaxLine( cOldSec ))
     FOR n := 1 TO nLines
       cOldLine := AllTrim( MemoLine( cOldSec, nLineWidth, n ))
       // comment lines begin with a non-letter character
       IF IsAlpha( Left( cOldLine, 1 )) .AND. ;
	  Left( cOldLine, Len( cItem )) == cItem
	  n := nLines
	  IF Empty( cOldValue := AfterAtNum( "=", cOldLine ))
	     // nothing to read; OldValue remains NIL
	  ELSE
	     // read value
	     OldValue := Str2Ini( cOldValue, cType )
	  ENDIF
	ENDIF
     NEXT
  ENDIF

ENDIF

RETURN OldValue  // remained NIL of item did not exist
// eof MidIniItem()
*****************************************************************************
STATIC FUNCTION Ini2Str( Value )

LOCAL cValue

IF     ValType( Value ) == "N"
  cValue := Str( Value )
ELSEIF ValType( Value ) == "C"
  cValue := Value
ELSEIF ValType( Value ) == "L"
  cValue := IF( Value, "Y", "N" )
ELSEIF ValType( Value ) == "D"
  cValue := DToC( Value )
ENDIF

RETURN cValue
// eof Ini2Str()
*****************************************************************************
STATIC FUNCTION Str2Ini( cValue, cType )

LOCAL Value

IF     cType == "N"
  Value := Val( cValue )
ELSEIF cType == "C"
  Value := cValue
ELSEIF cType == "L"
  Value := ( cValue == "Y" )
ELSEIF cType == "D"
  Value := CToD( cValue )
ENDIF

RETURN Value
// eof Str2Ini()
****************************************************************************
FUNCTION IniDelSection( cSecName )

LOCAL cSec := AfterAtNum( "[" + cSecName + "]", cIni    )
      cSec := BeforAtNum( "["                 , cSec, 1 )
      cSec := "[" + cSecName + "]" + cSec

      cIni := StrTran( cIni, cSec )

RETURN .T.
// eof DelIniSec()
*****************************************************************************
*****************************************************************************
// eof Ini.prg
